<?php
/*
 * Classe Template
 *  
 * @file         template.class.php
 * @author       MICHEL Sylvain
 * @version      1.0
 * @since        2002-09-01
 * @project      phpFullAnnu
 * @package      phpFullAnnu
 * @subpackage   I/O
 * @copyright    (C) 2002 MICHEL Sylvain webmaster@netsliver.com
 */

class Template {
	var $debug = FALSE;				//affiche les erreurs ou non
	var $file = array();			//tableau des fichiers template
	var $root = '';					//dossier des templates
	var $varkeys = array();			//nom des variables
    var $varvals = array();			//contenu des variables
	
	/*
	 * "remove" => efface les variables non dfinies
	 * "comment" => remplace les variables non dfinies par un commentaire
	 * "keep" => ne s'occupe pas des variables non dfinies 
	 */ 
	var $unknowns = 'remove';
	
	/*
	 * "yes" => stop sur toutes les erreurs
	 * "report" => affiche l'erreur mais continu
	 * "no" => ignore les erreurs 
	 */ 
	var $halt_on_error = 'yes';
	
	/*
	 * constructeur : dfini le rpertoire des templates,
	 * dfini ce qui est fait lorsqu'une variable est non dfinie
	 */ 
	function Template( $root = '.', $unknowns = 'remove' ) {
		$this->setRoot($root);
		$this->setUnknowns($unknowns);
	}
		
	/*
	 * dfini le rpertoire des templates.
	 */ 
	function setRoot( $root ) {
		if ( !is_dir($root) ) {
			$this->halt("$root isn't a directory.");
			return FALSE;
		}
		$this->root = $root;
		return TRUE;
	}
	
	/*
	 * dfini ce qui est fait pour les variables non dfinies ("remove", "comment" ou "keep")
	 */ 
	function setUnknowns( $unknowns = 'remove' ) {
		$this->unknowns = $unknowns;
	}
	
	/*
	 * dfini les noms de template <=> fichiers
	 */ 
	function setFile( $handle, $filename = '' ) {
		if ( !is_array($handle) ) {
			if ( empty($filename) ) {
				$this->halt("For handle $handle filename is empty.");
				return FALSE;
			}
			$this->file[$handle] = $this->filename($filename);
		} else {
			reset($handle);
			while ( list($h, $f) = each($handle) ) {
				$this->file[$h] = $this->filename($f);
			}
		}
	}
	
	/*
	 * dfini les variables du template
	 */ 
	function setVar( $varname, $value = '' ) {
		if ( !is_array($varname) ) {
			if ( !empty($varname) ) {
				if ( $this->debug ) {
					print "scalar: set *$varname* to *$value*<br>\n";
				}
			}
			$this->varkeys[$varname] = '/'.$this->varname($varname).'/';
			$this->varvals[$varname] = $value;
		} else {
			reset($varname);
			while ( list($k, $v) = each($varname) ) {
				if ( !empty($k) ) {
					if ( $this->debug ) {
						print "array: set *$k* to *$v*<br>\n";
					}
				}
				$this->varkeys[$k] = '/'.$this->varname($k).'/';
				$this->varvals[$k] = $v;
			}
		}
	}
	
	/*
	 * remplace une variable dans les templates
	 * par la valeur envoy par la fonction parse()
	 */ 
	function subst( $handle ) {
		if ( !$this->loadfile($handle) ) {
			$this->halt("Unable to load $handle.");
			return FALSE;
		}
		$str = $this->getVar($handle);
		$str = @preg_replace($this->varkeys, $this->varvals, $str);
		return $str;
	}
	
	/*
	 * remplace toutes les variables du template par les valeurs respectives
	 */ 
	function parse( $target, $handle, $append = FALSE ) {
		if ( !is_array($handle) ) {
			$str = $this->subst($handle);
			if ( $append ) {
				$this->setVar($target, $this->getVar($target).$str);
			} else {
				$this->setVar($target, $str);
			}
		} else {
			reset($handle);
			while ( list($i, $h) = each($handle) ) {
				$str = $this->subst($h);
				$this->setVar($target, $str);
			}
		}
		$str = $this->finish($str);
		return $str;
	}
	
	/*
	 * affiche le template avec les valeurs du script
	 */
	function pparse( $target, $handle, $append = FALSE ) {
		print $this->parse($target, $handle, $append);
		return FALSE;
	}
	
	/*
	 * retourne la valeur d'une variable pass en paramtre
	 */ 
	function getVar( $varname ) {
		if ( !is_array($varname) ) {
			return ( isset($this->varvals[$varname]) ) ? ($this->varvals[$varname]) : FALSE;
		} else {
			reset($varname);
			while ( list($k, $v) = each($varname) ) {
				$result[$k] = $this->varvals[$k];
			}
			return $result;
		}
	}
	
	/*
	 * s'occupe des variables non dfinies sur les templates
	 */
	function finish( $str ) {
		switch ( $this->unknowns ) {
			case 'keep':
			break;
			case 'remove':
				$str = preg_replace('/{[^ \t\r\n}]+}/', '', $str);
			break;
			case 'comment':
				$str = preg_replace('/{([^ \t\r\n}]+)}/', "<!-- Template $handle: Variable \\1 undefined -->", $str);
			break;
		}
		return $str;
	}
	
	/*
	 * gre le nom du fichier (s'il existe ou pas)
	 */
	function filename( $filename ) {
		if ( substr($filename, 0, 1) != '/' ) {
			$filename = $this->root."/".$filename;
		}
		if ( !file_exists($filename) ) {
			$this->halt("File $filename doesn't exist.");
		}
		return $filename;
	}
	
	/*
	 * gre les caratres spciaux inclus dans les noms de variables
	 */ 
	function varname( $varname ) {
		return preg_quote("{".$varname."}");
	}
	
	/*
	 * charge le fichier pour l'utiliser en tant que template dans la variable $str.
	 */
	function loadfile( $handle ) {
		if ( isset($this->varkeys[$handle]) && !empty($this->varvals[$handle]) ) {
			return TRUE;
		}
		if ( !isset($this->file[$handle]) ) {
			$this->halt("$handle is not a valid handle.");
			return FALSE;
		}
		$filename = $this->file[$handle];
		$str = implode('', @file($filename));
		if ( empty($str) ) {
			$this->halt("While loading $handle, $filename does not exist or is empty.");
			return FALSE;
		}
		$this->setVar($handle, $str);
		return TRUE;
	}
	
	/*
	 * gre l'affichage de messages d'erreur et l'arrt
	 */ 
	function halt( $msg ) {
		if ( $this->halt_on_error != 'no') {
			$this->haltmsg($msg);
		}
		if ( $this->halt_on_error == 'yes') {
			die("<b>Halted.</b>");
		}
		return FALSE;
	}
	
	/*
	 * affiche le message d'erreur
	 */ 
	function haltmsg( $msg ) {
		printf("<b>Template Error:</b> %s<br>\n", $msg);
	}
}
?>